/*******************************************************************************
* Copyright (C) 2005 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

// Author: Jon Carey (jcarey@novell.com)

#ifndef __LINUX_COMMON_HPP__
#define __LINUX_COMMON_HPP__

#include <openwbem/OW_String.hpp>
#include <openwbem/OW_CIMDateTime.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>

namespace OMC
{

/**
 * Get the class name that represents the operating system
 * @return A string that contains the class name that should be used
 * in OSCreationClassName properties.
 */
OpenWBEM::String getOperatingSystemClassName();

/**
 * Get the class name that represents the ComputerSystem.
 * @return A string that contains the class name that should be used
 * in CSCreationClassName properties.
 */
OpenWBEM::String getComputerSystemClassName();

/**
 * Get the common value used for the computer system.
 * @return A string that contains the computer system name. To be
 * used in values of the CSName property.
 */
OpenWBEM::String getComputerSystemName();

/**
 * Get the object path of the OMC_UnitaryComputerSystem
 * object.
 * @param ns The name space for the object.
 * @return The object path for the computer system object.
 */
OpenWBEM::CIMObjectPath getComputerSystemObjectPath(
	const OpenWBEM::String& ns);


/**
 * Get the object path of the OMC_OperatingSystem
 * object.
 * @param ns The name space for the object.
 * @return The object path for the computer system object.
 */
OpenWBEM::CIMObjectPath getOperatingSystemObjectPath(
    const OpenWBEM::String& ns);


/**
 * Determine if an object path points to the computer
 * system object path.
 * @param ns The namespace we're checking for.
 * @param cop The object path to check.
 * @return true if the object path points to the computer
 *		system object. Otherwise false.
 */
bool isComputerSystemObjectPath(
	const OpenWBEM::String& ns,
	const OpenWBEM::CIMObjectPath& cop);

/**
 * Get the common value used for the operating system.
 * @return A string that contains the OS name. To be used in values
 * of the OSName property.
 */
OpenWBEM::String getOperatingSystemName();

/**
 * Generate an instance id that has a prefix that is consistent
 * with all instance IDs used as keys for various NIM classes.
 * This would typically be used by providers of sub-classes
 * of CIM_SettingData, but it's use is not restricted to this.
 * The instance id will take the form of novell:suse:<uniquepart>
 * where uniquepart is provided by the caller.
 * @parm uniqueParm The unique part of the final instance id.
 * @return The instance id that will be used for a key value.
 */
OpenWBEM::String makeInstanceID(const OpenWBEM::String& uniquePart);

/**
 * Get the install date of an RPM as a CIMDateTime.
 * @param rpmName The name of the rpm to get the install date for.
 * @param pExitStatus If specified, the exit status of the RPM 
 *		command will be stored here.
 * @return A CIMDateTime object that represents the install date
 *		of the given rpm on success. Otherwise a NULL CIMDateTime
 *		object will be returned.
 */
OpenWBEM::CIMDateTime getRPMInstallDate(
	const OpenWBEM::String& rpmName,
	int *pExitStatus=NULL);



/**
 * Get the name of the interop namespace
 * @return A string that contains the name of the interop namespace
 * 
 */
OpenWBEM::String getInteropNamespaceName();

#define OMC_REG_PROV_EXPLICITLY_opt "omc.register_explicitly"

}	// End of namespace OMC

#endif	// __LINUX_COMMON_HPP__
