/*******************************************************************************
* Copyright (C) 2005 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

// Author: Jon Carey (jcarey@novell.com)

#ifndef __FILEFILTER_HPP__ 
#define __FILEFILTER_HPP__

#include <openwbem/OW_config.h>
#include <openwbem/OW_String.hpp>
#include <iostream>

namespace OMC
{

class FileModifierListener
{
public:
	virtual ~FileModifierListener() {}

	/**
	 * Called right after the file modifier opened the temp file
	 * for a modify operation. The file filter has not written
	 * anything to the file yet.
	 * @parm iostrm The is the stream for the tempfile that was
	 *		opened for modification.
	 * @return true if the listener wrote anything to the stream.
	 *		Otherwise false.
	 */
	virtual bool fileOpened(std::iostream& iostrm)
	{
		return false;
	}

	/**
	 * Called right before the file modifier closes the temp file
	 * for a modify operation.
	 * @parm iostrm The is the stream for the tempfile that was
	 *		opened for modification.
	 * @return true if the listener wrote anything to the stream.
	 *		Otherwise false.
	 */
	virtual bool fileClosing(std::iostream& iostrm)
	{
		return false;
	}

	/**
	 * Called right before the file modifier writes a line to the
	 * temp file for a modify operation.
	 * @parm line The line the file filter is going to write to
	 *		the temp file. The listener as the option of modifiying
	 *		this line.
	 * @parm ignoreLine if this is true on return, then the file
	 *		modifier will not write the line the file and will assume
	 *		a return value of true even if the function returns false
	 * @return true if the listener modified the given line parameter.
	 *		Otherwise false.
	 */
	virtual bool writingLine(OpenWBEM::String& line, bool& ignoreLine)
	{
		return false;
	}
};

class FileInspectorListener
{
public:
	virtual ~FileInspectorListener() {}

	/**
	 * Called right after the file inspector reads a line from
	 * the file it is processing.
	 * @parm line The line just read from the file
	 * @parm stopReading if this is set to true on return, the
	 *		the file inspector will stop processing the file.
	 */
	virtual void lineRead(const OpenWBEM::String& line, bool& stop) = 0;
};

class FileModifier
{
public:
	/**
	 * Constructor.
	 * @parm fileName The name of the file (absolute path) that will be
	 *		processed.
	 * @parm listener The listener that will be called while processing
	 *		the file.
	 */
	FileModifier(const OpenWBEM::String& fileName, 
		FileModifierListener* listener);

	/**
	 * Process the file specified in the constructor.
	 * @parm modified Will be set to true on return if the file was modified.
	 *		Otherwise it will be set to false.
	 * @parm errMsg Will be set to the error message if an error occurred
	 *		during processing.
	 * @return 0 on success. -1 if an error occurred.
	 */
	int processFile(bool& modified, OpenWBEM::String& errMsg);

	/**
	 * Move original file back in place. This is intended to reverse
	 * the changes made by processFile.
	 * @return true if the original file was found and put back in place.
	 *		Otherwise false.
	 */
	bool back2Orig();

private:
	OpenWBEM::String m_fileName;
	FileModifierListener *m_listener;
	bool m_modified;
};

class FileInspector
{
public:
	/**
	 * Constructor.
	 * @parm fileName The name of the file (absolute path) that will be
	 *		processed.
	 * @parm listener The listener that will be called while processing
	 *		the file.
	 */
	FileInspector(const OpenWBEM::String& fileName,
		FileInspectorListener* listener);

	/**
	 * Process the file specified in the constructor.
	 * @parm errMsg OUT - If an error occurred, a description of it will
	 *		be put in this String object.
	 * @return 0 on success. -1 if an error occurred.
	 */
	int processFile(OpenWBEM::String& errMsg);

private:
	OpenWBEM::String m_fileName;
	FileInspectorListener* m_listener;
};

}	// End of namespace OMC

#endif	// __FILEFILTER_HPP__
