/*******************************************************************************
* Copyright (C) 2005 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/
#include <openwbem/OW_config.h>
#include <openwbem/OW_String.hpp>
#include <openwbem/OW_Array.hpp>
#include <openwbem/OW_Exception.hpp>
#include <openwbem/OW_Types.hpp>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

OW_DECLARE_EXCEPTION(MountTable)

namespace OMC
{


class MountEntry
{
public:
    OpenWBEM::String getDevice() const
    {
        return _device;
    }
    OpenWBEM::String getMountPoint() const
    {
        return _mp;
    }
    OpenWBEM::String getType() const
    {
        return _type;
    }
    OpenWBEM::String getCreationClassName() const; 
    MountEntry(); 
    bool statGood() const
    {
        return _statGood;
    }
    bool hasOpt(const OpenWBEM::String& opt) const; 
    struct stat getStat() const
    {
        return _stat;
    }
    //operator bool(); 

private: 
    // man fstab: 
    // device, mountpoint, fstype, mntopts, fs_freq, fs_passno
    bool _statGood; 
    struct stat _stat; 
    OpenWBEM::String _device; 
    OpenWBEM::String _mp; 
    OpenWBEM::String _type; 
    OpenWBEM::StringArray _opts; 
    OpenWBEM::String _fs_freq; 
    OpenWBEM::String _fs_passno; 
    MountEntry(const OpenWBEM::String& mntent); 
    friend class MountTable; 
    bool _entryGood; 

private:
    // Fun hoops to jump through. 
    // "Any language that allows you to overload the comma operator should
    //  be shunned." -- Ted Tso, IBM.  LinuxWorld San Francisco, Aug 2003 
    struct dummy
    {
        void nonnull()
        {
        };
    };
    typedef void (dummy::*safe_bool)();
public:
    operator safe_bool () const
    {
        return _statGood ? &dummy::nonnull : 0;
    }
    safe_bool operator!() const
    {
        return _statGood ? 0: &dummy::nonnull;
    }
}; 

typedef OpenWBEM::Array<MountEntry> MountEntryArray; 

class MountTable
{
public:
    MountTable(); 
    MountEntryArray getMountEntries() const; 
    /** 
     * Return the name of the directory where the filesystem containing
     * the file is mounted. 
     * @param fname The filename in question
     * @param st A stat struct for fname (already filled out).
     * @param mt The mount table
     * @return the mount point
     */ 
    MountEntry getMountEntryForFile(const OpenWBEM::String& fname, 
                                    const struct stat& st) const; 

    MountEntry getMountEntryForFS(const OpenWBEM::String& device) const; 
private: 
    MountEntryArray _mpa; 
}; 

class FileTools
{
public:
    static void throwErrno(int err); 
    /**
     * Returns the name of the parent directory of a file name
     * @param arg The filename
     * @return The name of the parent directory
     */ 
    static OpenWBEM::String dirname(const OpenWBEM::String& fname); 

    /** 
     * Returns an ObjectPath for the given file. If you are going to be 
	 * calling this many times, you may want to use the 4 paremeter version. 
	 *
     * @param fname The filename
	 * @param ns The namespace 
     * @return The CIMObjectPath for the given file. 
     */
	static OpenWBEM::CIMObjectPath getObjectPathForFile( const OpenWBEM::String& fname, 
								 const OpenWBEM::String& ns); 
    /** 
     * Returns an ObjectPath for the given file. This function is more optimal
	 * than the two paramenter version because the mount table is built once, 
	 * and passed in by referance.  Otherwise the mount table will be created
	 * multiple times per file, and the file may be stat()ed multiple times. 
	 * 
     * @param fname The filename
     * @param st A stat struct already filled in for fname
     * @param mt A MountTable
	 * @param ns The namespace
     * @return The CIMObjectPath for the given file. 
     */
    static OpenWBEM::CIMObjectPath getObjectPathForFile(
                                                       const OpenWBEM::String& fname, 
                                                       const struct stat& st, const MountTable& mt, 
                                                       const OpenWBEM::String& ns); 
    static uid_t getEffectiveUID(const OpenWBEM::String& name); 
    static bool getUIDFromName(const OpenWBEM::String& name, uid_t& uid); 
    static bool getGIDFromName(const OpenWBEM::String& name, gid_t& gid); 
    static OpenWBEM::String getNameFromUID(uid_t uid); 
    static OpenWBEM::String getGroupFromGID(gid_t gid); 


}; 

}
