/*******************************************************************************
* Copyright (C) 2005 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

// Author: Jon Carey (jcarey@novell.com)

#ifndef __LINUXPROCESS_HPP__
#define __LINUXPROCESS_HPP__

#include <ctime>

#include <signal.h>

#include <openwbem/OW_Array.hpp>
#include <openwbem/OW_CIMDateTime.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_String.hpp>
#include <openwbem/OW_Map.hpp>

namespace OMC
{

class LinuxProcess;		// Fwd Ref

typedef OpenWBEM::Map<OpenWBEM::String, OpenWBEM::String> EnvMap;
typedef OpenWBEM::Map<int, OpenWBEM::String> FileHandleMap;
typedef OpenWBEM::Array<LinuxProcess> LinuxProcArray;

typedef struct {
	OpenWBEM::String 		execName;
	OpenWBEM::UInt64      	kernelModeTime;
	OpenWBEM::UInt32      	niceValue;
	OpenWBEM::UInt32      	parentProcessID;
	OpenWBEM::Real32      	percentCPU;
	OpenWBEM::UInt32      	priority;
	OpenWBEM::UInt32      	processGroup;
	OpenWBEM::UInt64      	sessionID;
	OpenWBEM::CIMDateTime 	startTime;
	OpenWBEM::UInt16      	state;
	OpenWBEM::UInt32      	ttyNumber;
	OpenWBEM::UInt64      	userModeTime;
	OpenWBEM::UInt32		virtualMemSize;
	OpenWBEM::UInt32		residentSetSize;
	OpenWBEM::UInt32		sharedMemSize;
} process_stats_t;

class LinuxProcess
{
public:

	enum ProcState
	{
		EPROCSTATE_DoesntExist = -1,		// Process doesn't exist
		EPROCSTATE_Unknown = 0,
		EPROCSTATE_Other = 1,
		EPROCSTATE_Ready = 2,
		EPROCSTATE_Running = 3,
		EPROCSTATE_Blocked = 4,
		EPROCSTATE_SuspendedBlocked = 5,
		EPROCSTATE_SuspendedReady = 6,
		EPROCSTATE_Terminated = 7,
		EPROCSTATE_Stopped = 8,
		EPROCSTATE_Growing = 9
	};

	/**
	 * Construct a new LinuxProcess object with a pid of 0
	 */
	LinuxProcess() : m_pid(0) {}

	/**
	 * Construct a new LinuxProcess object
	 * @param pid The process ID this LinuxProcess object represents.
	 */
	LinuxProcess(int pid);

	/**
	 * Construct a new LinuxProcess object that is a copy of another.
	 * @param procArg The LinuxProcess object this object will be a
	 * 		copy of.
	 */
	LinuxProcess(const LinuxProcess& procArg)
		: m_pid(procArg.m_pid) {}

	/**
	 * Construct a new LinuxProcess object that represents the process id
	 * contained in the given pid file.
	 * @param pidFile The fully qualified path to the pid file
	 */
	LinuxProcess(const OpenWBEM::String& pidFile);

	/**
	 * Assignment operator
	 * @param arg The LinuxProcess object to assign to this one.
	 * @return A reference to this LinuxProcess object.
	 */
	LinuxProcess& operator= (const LinuxProcess& arg)
	{
		m_pid = arg.m_pid;
		return *this;
	}

	/**
	 * Kill the process associated with this LinuxProcess object.
	 * @param sig The signal to be sent to the process. Defaults to KILL
	 * @return 0 on success. -1 on failure
	 */
	int kill(int sig=SIGKILL);

	/**
	 * @return The process id.
	 */
	int getProcessID() const { return m_pid; }

	/**
	 * @return The CIM object path for this process
	 */
	OpenWBEM::CIMObjectPath getObjectPath(const OpenWBEM::String& ns) const;

	/**
	 * @return the name of the file being executed.
	 * Note this is not a fully qualified path.
	 */
	OpenWBEM::String getExecName() const;

	/**
	 * @return The current state of the process
	 */
	ProcState getState() const;

	/**
	 * @return The Parent process ID
	 */
	int getParentProcessId()const;

	/**
	 * @return The process group ID
	 */
	int getProcessGroup() const;

	/**
	 * @return The session ID of the process
	 */
	int getSessionID() const;

	/**
	 * @return the real user id of the process
	 */
	unsigned long long getRealUID() const;

	/**
	 * @return The number of the TTY the process uses
	 */
	int getTTYNumber() const;

	/**
	 * @return The process group ID of the process which currently owns
	 * the tty that this process is connected to.
	 */
	int getOwningPGRPOfTTY() const;

	/**
	 * @return The flags of the process. The math bit is decimal 4 and
	 * the traced bit is decimal 10.
	 */
	unsigned long getFlags() const;

	/**
	 * @return The standard nice value.
	 */
	long getPriority() const;

	/**
	 * @return The nice value. This ranges from 19 (nicest) to -19
	 * (not nice at all)
	 */
	long getNiceValue() const;

	/**
	 * @return The time the process started.
	 */
	time_t getStartTime() const;

	/**
	 * @return The fully qualified path to the executible
	 */
	OpenWBEM::String getModulePath() const;

	/**
	 * Get the all command line components as a string array.
	 * @param args A string array where the command line
	 * 		components will be placed
	 * @return 0 on success. -1 on failure.
	 */
	int getCommandLine(OpenWBEM::StringArray& args) const;

	/**
	 * @return Kernel Mode time in ms
	 */
	unsigned long long getKernelModeTime() const;

	/**
	 * @return User Mode time in ms
	 */
	unsigned long long getUserModeTime() const;

	/**
	 * Get the environment variables being used by this process
	 * @param envMap A string map that will contain the environment
	 * 		for the process.
	 * @return 0 on success. -1 on failure.
	 */
	int getEnv(EnvMap& envMap) const;

	/**
	 * Get the current working directory of the process.
	 * @return The cwd as a string object.
	 */
	OpenWBEM::String getCwd() const;

	/**
	 * Get the root of the file system for the process.
	 * If the process is chroot'ed then this method will return that dir name.
	 * @return The root of the file system for the process as a string.
	 */
	OpenWBEM::String getRoot() const;

	/**
	 * @return The virtual memory size of the process in bytes
	 */
	unsigned long getVirtualMemSize() const;

	/**
	 * @return the memory resident set size
	 */
	unsigned long getResidentSetSize() const;

	/**
	 * @return the shared memory size
	 */
	unsigned long getSharedMemSize() const;

	double getPercentCPU(bool includeDeadChildren=false) const;

	/**
	 * Get the open files for the process.
	 * @param fmArg A FileHandleMap that will contain the open files for
	 * 		the process on return
	 * @return 0 on success. -1 on failure
	 */
	int getOpenFiles(FileHandleMap& fmArg) const;

	/**
	 * @return The time the system came up.
	 */
	static time_t getSystemBootTime();

	/**
	 * Get the information out of the /proc/<pid>/stat file
	 */
	process_stats_t getProcStats();

	/**
	 * Get processes.
	 * @parm procsArg All process objects found will be placed in this array.
	 * @parm regexPattern This this parameter is specified, it is expected to
	 *		be a valid extended regular expression. Only processes with an
	 *		exec name that matches this pattern will be returned.
	 * @return 0 on success. -1 on failure.
	 */
	static int getAllProcs(LinuxProcArray& procsArg, 
		const char* regexPattern=0);

	/**
	 * Determine if a pid file is valid and contains a process id of a
	 * a process with a given exec name.
	 * @parm pidFileName The name of the pid file to check for validity
	 * @parm execName The exec name the process should have.
	 * @return true if the pid file is valid. Otherwise false.
	 */
	 static bool validPidFile(const OpenWBEM::String& pidFileName, 
	 	const OpenWBEM::String& execName);

	/**
	 * Determine if a given object path is for a process that has
	 * a given exec name.
	 * @parm cop The object path to check.
	 * @parm execName The exec name the process should have.
	 * @return true if the object path is for a process and the
	 *		process's exec name matches the given exec name. Otherwise
	 *		return false.
	 */
	static bool opIsProcessAndExec(const OpenWBEM::CIMObjectPath cop,
		const OpenWBEM::String execName);

private:

	static ProcState _char2ProcState(char c);
	static int _getTaskState(int pid, int tid);
	static ProcState _stateThruTasks(int pid);

	static time_t _system_boot_time;
	
	static FILE* _openStatFile(int pid);
	static FILE* _openMemFile(int pid);

	int m_pid;
};

}	// End of namespace OMC

#endif	// __LINUXPROCESS_HPP__
