/*******************************************************************************
* Copyright (C) 2005 Novell, Inc. All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*
*  - Redistributions of source code must retain the above copyright notice,
*    this list of conditions and the following disclaimer.
*
*  - Redistributions in binary form must reproduce the above copyright notice,
*    this list of conditions and the following disclaimer in the documentation
*    and/or other materials provided with the distribution.
*
*  - Neither the name of Novell, Inc. nor the names of its
*    contributors may be used to endorse or promote products derived from this
*    software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
* AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
* ARE DISCLAIMED. IN NO EVENT SHALL Novell, Inc. OR THE CONTRIBUTORS
* BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
* CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
* SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
* INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
******************************************************************************/

// Author: Jon Carey (jcarey@novell.com)

#ifndef PROVIDERSTORE_HPP_GUARD
#define PROVIDERSTORE_HPP_GUARD

#include <openwbem/OW_String.hpp>
#include <openwbem/OW_CIMObjectPath.hpp>
#include <openwbem/OW_Map.hpp>
#include <openwbem/OW_CIMException.hpp>
#include <openwbem/OW_ProviderEnvironmentIFC.hpp>

extern "C"
{
#include <strings.h>
}

namespace OMC
{

using namespace OW_NAMESPACE;

OW_DECLARE_EXCEPTION(ProviderStore);

// Compare to create a case insensative string map
struct ProviderStorePropsCompare
{
	bool operator()(const String& k1, const String& k2)
	{
		return ::strcasecmp(k1.c_str(), k2.c_str()) < 0;
	}
};

typedef Map<String, String, ProviderStorePropsCompare> ProviderStoreProps;

/**
 * The ProviderStore class is a set of static methods that handle the storage
 * of implementation specific data for providers in the /omc/internal
 * namespace of the OpenWBEM CIMOM.
 * The intention is for the stored data to correspond with an instance in
 * another namespace that a provider is instrumenting in. Thus the stored
 * data is keyed off of the namespace of the caller plus the class name
 * and any keys associated with the instance in the form of a CIMObjectPath.
 */
class ProviderStore
{
public:
	/**
	 * Store some properties for an instance that resides in a given namespace.
	 * @param env A ProviderEnvironmentIFCRef that the store procedure
	 *		will use to process the given request.
	 * @param ns The namespace for the instance that this data is being
	 * 		store for.
	 * @param key The CIMObjectPath that references the instance this data
	 * 		is being stored for.
	 * @param props The Properties and Values to be stored for the instance
	 * 		referenced by the 'key' param.
	 * Throws ProviderStoreException if there is a failure of any sort.
	 * Throws CIMException if some failure was encountered with the CIMOM.
	 */
	static void storeProps(
		const ProviderEnvironmentIFCRef& env,
		const String& ns,
		const CIMObjectPath& key,
		const ProviderStoreProps& props);

	/**
	 * Get some properties for an instance that resides in a given namespace.
	 * @param env A ProviderEnvironmentIFCRef that the get procedure
	 *		will use to process the given request.
	 * @param ns The namespace for the instance that this data is being
	 * 		retrieved for.
	 * @param key The CIMObjectPath that references the instance this data
	 * 		is being retrieved for.
	 * @return A ProviderStoreProps object that contains the properties and 
	 * 		values that were previously stored for the given instance.
	 * Throws ProviderStoreException if there is a failure of any sort.
	 * Throws CIMException if some failure was encountered with the CIMOM.
	 */
	static ProviderStoreProps getProps(
		const ProviderEnvironmentIFCRef& env,
		const String& ns,
		const CIMObjectPath& key);
private:
	ProviderStore() {}	// Forbid creation of instances
};

}	// End of namespace namespace


#endif	// PROVIDERSTORE_HPP_GUARD
